importPackage(java.lang);
importPackage(java.util);
importPackage(java.io);
importPackage(java.text);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessage);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLMessageImpl);
importClass(Packages.com.alarmpoint.integrationagent.apxml.APXMLToken);
importClass(Packages.com.alarmpoint.integrationagent.security.EncryptionUtils);
importClass(Packages.com.thoughtworks.xstream.XStream);
importClass(Packages.com.thoughtworks.xstream.converters.reflection.PureJavaReflectionProvider);

importClass(Packages.org.apache.commons.httpclient.Header);
importClass(Packages.org.apache.commons.httpclient.HttpVersion);
importClass(Packages.org.mule.providers.http.HttpResponse);

var xStream = new XStream(new PureJavaReflectionProvider())

load("integrationservices/hpomi20/lib/baseclass.js");
load("integrationservices/hpomi20/lib/logger.js");
load("integrationservices/hpomi20/lib/wsutil.js");
load("integrationservices/hpomi20/lib/xmattersws.js");

load("integrationservices/hpomi20/omi-config.js");
load("integrationservices/hpomi20/hpomi-request.js");
load("integrationservices/hpomi20/hpomi-rest.js");


var exceptionString = "JavaException: ";

var log = new Logger();
var preamble = "[hpomi20] ";

var notePrefix = "[xMatters] - "; // The Prefix which is added to the note comments e.g. "[xMatters] " 

var OMI_PASSWORD = getPassword(OMI_PASSWORD_FILE);

/**
 * This is the injection point to xMatters from OMi Integration Connected Server
 *  
 * This method will parse the OMi request and generate an APXML Message to be 
 * sent to xMatters via the ServiceAPI.
 *
 * Modify the apxml object to add tokens injected to xMatters.
 */
function apia_http( httpRequestProperties, httpResponse )
{
    log.debug("Received a request from HP OMi with httpRequestProperties: " + httpRequestProperties);
    var httpRequest = httpRequestProperties.getProperty("http.request");
    
    body = httpRequestProperties.getProperty("REQUEST_BODY") +"";
    log.debug( "Body: " + body );
    
    var payload = new XML( new WSUtil().formatStringForE4X( body ) );
    
    // Create handler for understanding the OMi request.
    var omiRequest = new HPOMIRequest();
    
    // Opr Event Change objects only contain what fields have changed.  Get the entire details of the Event to send to xMatters.
    if(httpRequest.contains("/event_change/"))
    {
      // payload should be an OMi Opr Event Change Object
      var eventId = httpRequest.substring(httpRequest.lastIndexOf("/") + 1);
      log.debug("Handling an event_change request for eventID: " + eventId);

      // query OMi for the entire event record
      var restService = new HPOMIRestService(OMI_PROTOCOL, OMI_SERVER, OMI_PORT, OMI_REST_SYNC_EVENT_ROOTPATH, OMI_USER, OMI_PASSWORD);
      log.debug("Requesting event details from HP OMi Rest web service for eventId: " + eventId);
      var response = new XML( new WSUtil().formatStringForE4X( restService.get(eventId)+"" ) );
      log.debug("    Rest web service request was successful for eventId: " + eventId);
      //  if uncommented, the following logdebug statement will add a great deal of bulk to the IA Log
      // log.debug("Received the following event details from HP OMi: " + response.toXMLString());
      
      // Overwrite the Opr Event Change object with the more detailed Opr Event object
      payload = response;  
    }

    // payload should be an OMi Opr Event Object      
    var apxml = omiRequest.convertOprEventToAPXML(payload);     
    
    // Send event to xMatters and delete any existing events with that incident/event id.
    var deleteExistingEvents = true;
    log.debug("Preparing to send APXML to xMatters server with deleteExistingEvents flag: " + deleteExistingEvents);
    new xMattersWS().submitAPXML(apxml, deleteExistingEvents);

    // Build Response to OMi.  At a minimum the response must be either an Opr Event object or an OprEventChange object
    log.debug("Preparing to send a response to OMi...");
    var responseDoc;
    if(httpRequest.contains("/event_change/"))
    {
      responseDoc = omiRequest.makeEventChangeRequest(apxml.getValue("incident_id"));
    }
    else
    {
      responseDoc = omiRequest.makeEventDoc(apxml.getValue("incident_id"));
    }
    httpResponse.setBodyString(responseDoc);          
    httpResponse.setHeader(new Header("Content-Type", httpRequestProperties.getProperty("Content-Type")));
    log.debug("Status of response sent to OMi: " + httpResponse.getStatusLine());
    return httpResponse;
}

/**
 * This is the main method for handling APXML messages sent to the 
 * Integration Service by APClient.  The messages may be requests to perform 
 * local activity, or they may be requests to make submissions to AlarmPoint.
 * <p>
 * Any APXMLMessage object that this method returns will be sent to xMatters
 * via the Integration Service's outbound queues.
 */
function apia_input(apxml)
{
  // Forward APXML to xMatters.
  return apxml;  
}

/**
 * This is the main method for handling APXML messages sent to the
 * Integration Service by AlarmPoint.  The messages may be responses to 
 * previous submissions, or they may be requests to the Integration Service
 * that originate from AlarmPoint 
 * (e.g., via an ExternalServiceMessage/ExternalServiceRequest2).
 * <p>
 * Any APXMLMessage object that this method returns will be sent to AlarmPoint
 * either via the Integration Service's outbound queues or directly as a 
 * Web Service response, depending on the mechanism used to submit the APXML.
 */
function apia_response(apxml) 
{
  var method = apxml.getMethod();
  
  log.info("Received APXML from xMatters with method: " + method);
  if (APXMLMessage.OK_METHOD.equalsIgnoreCase(method))
  {
    return handleOK(apxml);
  } 
  else if (APXMLMessage.ERROR_METHOD.equalsIgnoreCase(method))
  {
    return handleERROR(apxml);
  } 
  else if (APXMLMessage.SEND_METHOD.equalsIgnoreCase(method))
  {
    return handleSend(apxml);
  }
  else if (APXMLMessage.REQUEST_METHOD.equalsIgnoreCase(method))
  {
    return handleRequest(apxml);
  }
  else
  {
    throw new IllegalArgumentException("Unrecognized APXML method: " + method);
  }
}

/**
 * OK responses are returned by AlarmPoint for each APXML message that it accepts
 * from an Integration Service.
 * <p>
 * Log the acknowledgement using the nnmi Integration Service's logger.
 */
function handleOK(apxml)
{
  var id = apxml.getTransactionId();
  log.info("  Submission of transaction " + id + " succeeded.");
}

/**
 * ERROR responses are returned by AlarmPoint for each APXML message that it
 * is unable to accept from an Integration Service.
 * <p>
 * Log the error using the nnmi Integration Service's logger.
 */
function handleERROR(apxml) 
{
  var id = apxml.getTransactionId();
  var code = apxml.getValue(APXMLMessage.ERROR_CODE); 
  log.error("  Submission of transaction " + id + "failed with error code " + code);
}

/**
 * Requests are forwarded to the Send handler.
 * <p>
 * Redirects to the handleSend as messages may come from ESM or ESR to the IA.
 */
function handleRequest(apxml)
{
  log.debug( "  handleRequest received apxml, forwarding to handleSend()..." );
  return handleSend(apxml);
}

/**
 * The main routine for handling Requests from AlarmPoint
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleSend(apxml)
{
  log.debug( "    handleSend() - APXML message has been received from callout APS script. " );

  // handle annotations coming from callout APS script 
  // they are formatted differently 
  // the presence of the message token means this message 
  // is a callout annotation.
  var message = apxml.getValue("message");
  var incidentId = apxml.getToken("incident_id").getValue();
  
  if ( message != null && incidentId != null)
  {
    return handleCalloutAnnotate( apxml );
  }
  else
  {
    return handleApsResponses( apxml );
  }
}
 
/**
 * Used to update HP OMi using REST web services
 * <p>
 * The main routine for handling user responses to xMatters notifications.
 * <p>
 * Throw an exception that will be logged by the Integration Agent.
 */
function handleApsResponses(apxml) 
{
  // deserialize the additional tokens
  var tokens = getTokens(apxml);
  log.debug( "      APXML from xMatters includes these Tokens: " + tokens);
  
  var incidentId = tokens.get("incidentId");
  var responseEvent = tokens.get("responseEvent");
  var responseAction = tokens.get("responseAction");
  var annotation = tokens.get("annotation");
  var responder = tokens.get("responder");
  var personTarget = tokens.get( "personTarget" );

  // check for required tokens.
  if(responseEvent == "RECEIVED_RESPONSE" || responseEvent == "STANDALONE_ANNOTATION")
  {
    // For RECEIVED_RESPONSE events, a responseAction key/value pair should exist.
    if(responseAction == null || responseAction == "" || responseAction == undefined)
    {
      throw "responseAction not set for a RECEIVED_RESPONSE responseEvent.  responseAction required check AlarmPoint scripting to ensure this token is set";
    }
  }
  else if(responseEvent == "SUCCESSFUL_DELIVERY" || responseEvent == "DELIVERY_FAILURE")
  {
    // Create an annotation for the successful or failed delivery of a notification.
    annotation = notePrefix + responseEvent + " to " + responder;
    
    // Set the responseAction token to Annotate to pass control to the Annotate handler.
    responseAction = "Annotate";
    
    if(!ANNOTATE_DELIVERY)
    {
      log.warn("      ANNOTATE_DELIVERY flag is FALSE, so the following message will not be annotated to OMi event: [" + incidentId + "] " + annotation);
      return;
    }
  }
  else if(responseEvent == "DEVICE_DELIVERED")
  {
    // Log voice device delivery, but do not annotate to OMi, a callout annotation will be used instead.
    // So just return.
    log.debug("      APS Response was " + responseEvent + " to "  + responder + " - no annotation will be sent to OMi.");
    return;
  }
  else
  {
    throw "Unknown responseEvent.  Expected: RECEIVED_RESPONSE||SUCCESSFUL_DELIVERY||DELIVERY_FAILURE||DEVICE_DELIVERED. Received: " + responseEvent;
  }

  // Initialize xMattersWS interface building a reply to xMatters.
  var xmWebService = new xMattersWS();
  var xmResponse = xmWebService.initializeResponse(apxml);
        
  log.debug("      Preparing OMi REST web service request for response action [" + responseAction +"]");
  try
  {
    // Get the handler for creating an OMi Event Change Request
    var omiRequest = new HPOMIRequest();

    // Create the request body setting the id of the event to be updated
    var eventChange = omiRequest.makeEventChangeRequest(incidentId);
    
    if ( responseAction.equalsIgnoreCase( "annotate" ) )
    {
      // Add annotation text to the Event Change Request.
      log.debug("        Annotation is: " + annotation);
      eventChange = omiRequest.addChangeProperty(eventChange, omiRequest.makeAnnotationElement(annotation));
    }
    else if ( responseAction.equalsIgnoreCase( "ignore" ) )
    {
      // Add ignore text to the Event Change Request.
      var ignoreMsg = notePrefix + "Event ignored by '" + responder;
      log.debug("        Annotation is: " + ignoreMsg);
      eventChange = omiRequest.addChangeProperty(eventChange, omiRequest.makeAnnotationElement(ignoreMsg));    
    }    
    else if ( responseAction.equalsIgnoreCase( "open" ) ||
              responseAction.equalsIgnoreCase( "in_progress" ) ||
              responseAction.equalsIgnoreCase( "resolved" ) ||
              responseAction.equalsIgnoreCase( "closed" ) )
    {
      // Add annotation text and new state to the Event Change Request.
      var stateChangeMsg = notePrefix + "State of event changed to '" + responseAction + "' by " + responder;
      if(annotation != null && annotation != "" && annotation != undefined)
      {
        eventChange = omiRequest.addChangeProperty(eventChange, omiRequest.makeAnnotationElement(annotation));
      }
      eventChange = omiRequest.addChangeProperty(eventChange, omiRequest.makeAnnotationElement( stateChangeMsg ) );
      eventChange = omiRequest.addChangeProperty(eventChange, omiRequest.makeStateElement( responseAction ) );
      eventChange = omiRequest.addChangeProperty(eventChange, omiRequest.makeUserElement( personTarget ) );
      
    }
    else if ( responseAction.equalsIgnoreCase( "critical" ) ||
              responseAction.equalsIgnoreCase( "major" ) ||
              responseAction.equalsIgnoreCase( "minor" ) ||
              responseAction.equalsIgnoreCase( "warning" ) ||
              responseAction.equalsIgnoreCase( "normal" ) )
    {            
      // Add annotation text and new severity to the Event Change Request.
      var sevChangeMsg = notePrefix + "Severity of event changed to '" + responseAction + "' by " + responder;
      if(annotation != null && annotation != "" && annotation != undefined)
      {
        eventChange = omiRequest.addChangeProperty(eventChange, omiRequest.makeAnnotationElement(annotation));
      }            
      eventChange = omiRequest.addChangeProperty(eventChange, omiRequest.makeAnnotationElement(sevChangeMsg));
      eventChange = omiRequest.addChangeProperty(eventChange, omiRequest.makeSeverityElement(responseAction));
    }
    else
    {
      throw "Unknown responseAction: " + responseAction;    
    }

    // Send the update to OMi
    var restService = new HPOMIRestService(OMI_PROTOCOL, OMI_SERVER, OMI_PORT, OMI_REST_SYNC_EVENT_CHANGE_ROOTPATH, OMI_USER, OMI_PASSWORD);        
    log.debug("      About to send request to OMi REST web service for [" + responseAction + "] request..." );
    var result = restService.post(eventChange, incidentId);
    log.debug("      Received the following response from OMi REST web service: " + result);
    xmResponse.setToken(xmWebService.TOKEN_NAME, "OK", APXMLToken.Type.STRING);
  }
  catch (e)
  {
    var errMsg = "Caught exception processing [" + responseEvent + "|" + responseAction + "]: " + e;
    log.error(errMsg);
    xmResponse.setToken(xmWebService.TOKEN_NAME_EXCEPTION, errMsg.replace(exceptionString, ""), APXMLToken.Type.STRING);
  }
  return xmResponse;
}


/**
 * handles annotations sent from the Callout APS script
 */
function handleCalloutAnnotate( apxml )
{    
  log.debug( "      Preparing to send annotation to OMi based on APS from xMatters...");
  var messageText = apxml.getValue("message");
  var incidentId = apxml.getValue("incident_id");

  if(!ANNOTATE_DELIVERY)
  {
    log.warn("      ANNOTATE_DELIVERY flag is FALSE, so the following message will not be annotated to OMi event: [" + incidentId + "] " + messageText);
    return;
  }
  
  log.debug("      Preparing OMi REST web service annotation request for incidentId [" + incidentId +"]");

  try 
  {
    var omiRequest = new HPOMIRequest();
    var eventChange = omiRequest.makeEventChangeRequest(incidentId);
    eventChange = omiRequest.addChangeProperty(eventChange, omiRequest.makeAnnotationElement(notePrefix + messageText));
    
    var restService = new HPOMIRestService(OMI_PROTOCOL, OMI_SERVER, OMI_PORT, OMI_REST_SYNC_EVENT_CHANGE_ROOTPATH, OMI_USER, OMI_PASSWORD);
    var result = restService.post(eventChange, incidentId);
    log.debug("            Received the following response from OMi REST web service: " + result);
  }
  catch (e)
  {
    log.error("Caught exception processing callout annotate for [" + incidentId + "] Exception:" + e);
    throw e;
  }
}

/**
 * Retrieves the additional tokens from the APXML
 * @param apxml XML message to process
 * @return additional tokens
 * @throws if the additional tokens are missing from the XML.
 */
function getTokens(apxml)
{
  if (apxml.getValue("additionaltokens") != null && !"".equalsIgnoreCase(apxml.getValue("additionaltokens"))) 
  {
    return xStream.fromXML(apxml.getValue("additionaltokens")); // LinkedHashMap
  }
  throw new Exception("additionaltokens is missing from apxml"); 
}

/**
 * Decrypts the password for the ServiceDeskUser used by the integration
 * @param passwordFile file/path of the file containing the encrypted password
 * @return decrypted password or an empty string if the password cannot be decrypted
 */
function getPassword(passwordFile)
{
  try
  {
    var encryptionUtils = new EncryptionUtils();
    var file = new File(passwordFile);
    return encryptionUtils.decrypt(file);
  }
  catch (e)
  {
    return "";
  }
}